HTTP_STATUS_OK = 200
HTTP_STATUS_MOVED = 302
HTTP_STATUS_BAD_REQUEST = 400
HTTP_STATUS_FORBIDDEN = 403
HTTP_STATUS_NOT_FOUND = 404

HTTP_DEFAULT_PORT = 80
HTTPS_DEFAULT_PORT = 443

URL_EXPIRATION_IN_SECONDS = 86400
MAX_AUTH_ATTEMPTS = 2

BUSINESS_OWNER_PROFILE_ID = 'BusinessProfile'
SBCS_ID = 'SmallBusinessCreditShare'
BUSINESS_PREMIER_PROFILE_ID = 'PremierProfile'

TERMS_RESPONSE_CODE = '031'
